package encryptiondemo;
import java.math.*;

/**
 * <p>Title: Process Reporter</p>
 * <p>Description: Generates a report, which the applet can then display.  This allows the applet presentation to change independant of the content</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * @author Thomas J. Fitzsimmons
 * @version 1.0
 */

public class ProcessReporter {

   private BigInteger p;
   private BigInteger q;
   private long e;
   private BigInteger d;
   private BigInteger n;
   private BigInteger nd;
   private String message;
   private String[] processes;
   private int currentIndex;


   public ProcessReporter(String p, String q, String e, String message) {
      this.p = new BigInteger(p);
      this.q = new BigInteger(q);
      this.e = Integer.parseInt(e);
      this.d = Encryption.calcD(new BigInteger(e), this.p, this.q);
      this.message = message;
      this.currentIndex = 1;
      this.n = this.p.multiply(this.q);
      this.nd = this.p.subtract(new BigInteger("" + 1)).multiply(this.q.subtract(new BigInteger("" + 1)));
      processes = new String[20];
      createProcesses();
   }

   public String getNext(){
      if( !(currentIndex > 9) ){
         return processes[currentIndex++];
      }
      if ( currentIndex == 10){
         return processes[currentIndex++];
      }
      return processes[currentIndex-1];
   }

   public String getPrev(){
      /*if( (currentIndex > 9) ){
         return processes[currentIndex-1];
      }*/
      if( !(currentIndex < 3) ){
         currentIndex--;
         return processes[currentIndex-1];
      }
      if( !(currentIndex < 2) ){
         return processes[--currentIndex];
      }
      return processes[currentIndex];
   }


   private void createProcesses(){
      char xChar = ' ';
      int xInt = 0;
      BigInteger decInt;
      BigInteger encInt;
      String encMessage;
      String spacer;

      spacer = "                       ";
      processes[0] = spacer + "Variable Values" + spacer;

      processes[1] = processes[0] +
                     "\n\nP =       " + p +
                     "\nQ =       " + q +
                     "\nE =       " + e;

      processes[2] = processes[0] +
                     "\n\nLet N = p*q = " + n;

      processes[3] = processes[0] +
                     "\n\nYour \"Public Key\" is now E and N,\nwhich you can give to anyone.";

      processes[4] = processes[0] +
                     "\n\nLet S = (p-1)(q-1) = " + nd +
                     "\n\nS is needed to compute your private key,\nand should be given to NO-ONE!";

      processes[5] = processes[0] +
                     "\n\nLet d represent your \"Private key\"." +
                     "\nThis will be the key which you use to\ndecode messages sent to you." +
                     "\n\nIt is difficult to calculate d, you\nhave to find a value of d such that:"+
                     "\n(e x d) mod S = 1" +
                     "\n\nIn this case d = " + d;

      xChar = message.charAt(0);
      xInt = (new Character(xChar)).hashCode();
      spacer = "                   ";


      processes[6] = spacer + "Letters to Numbers" +
                     "\n\nWe now need to convert the letters in the\nmessage to numbers so they can be\nencoded\n" +
                     "\nFirst letter of the message is: " + xChar +
                     "\n\nLetter " + xChar + " is equivelent to the decimal\nnumber " +
                     xInt + " using the ASCII ecoding\nsystem.";

      encInt = Encryption.encrypt(new BigInteger("" + xInt) , new BigInteger("" + e), n);

      processes[7] = spacer + "        Encryption" +
                     "\n\nLet 'C' represent encrypted number\n" +
                     "\nLet 'M' represent original number\n"+
                     "\nM = " + xInt +"\n\n" +
                     "C = (M^e) mod(N)\n"+
                     "C = (" + xInt + "^" + e + ") mod(" + n + ")\n"+
                     "C = " + encInt;

      encMessage = EncryptString.encryptString(message, e, n);

      processes[8] = spacer + "        Encryption" +
                     "\n\nThe rest of the characters in the code are\n" +
                     "processed in the same way, to give the\nfollowing string:" +
                     "\n\nMessage in = " + message +
                      "\nMessage out = " + encMessage;

      decInt = Encryption.decrypt(encInt, d, n);

      processes[9] = spacer + "        Decryption" +
                     "\n\nLet 'C' reprsent encrypted number\n" +
                     "Let 'M' represent the original number\n"+
                     "\nC = " + encInt +
                     "\n\nM = (C^e) mod(N)\n"+
                     "M = (" + encInt + "^" + d + ") mod(" + n + ")\n"+
                     "M = " + decInt;

      processes[10] = spacer + "        Decryption" +
                     "\n\nThe rest of the characters in the code are\n" +
                     "processed in the same way, to give the\nfollowing string:" +
                     "\n\nMessage in = " + encMessage +
                      "\nMessage out = " + EncryptString.decryptString(encMessage, d, n);

   }

}