package encryptiondemo;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.math.*;
import javax.swing.*;

/**
 * <p>Title: Display Applet</p>
 * <p>Description: This is the "Front-end" presentation of the RSA demonstration</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * @author Thomas J. Fitzsimmons
 * @version 1.4
 */

public class displayApplet extends Applet{


   private boolean isStandalone = true;
   TextDisplay textDisplay;
   InputPanel inputPanel;


   //Get a parameter value
   public String getParameter(String key, String def) {
      return isStandalone ? System.getProperty(key, def) :
         (getParameter(key) != null ? getParameter(key) : def);
   }

   //Construct the applet
   public displayApplet() {
   }
   //Initialize the applet
   public void init() {
      try {
         jbInit();
      }
      catch(Exception e) {
         e.printStackTrace();
      }
   }
   //Component initialization
   private void jbInit() throws Exception {

      textDisplay = new TextDisplay();
      inputPanel = new InputPanel();

      setLayout(new BorderLayout());
      add("North", new ControlPanel(this, textDisplay, inputPanel));

      add("Center", textDisplay);

      add("South", inputPanel);


   }
   //Get Applet information
   public String getAppletInfo() {
      return "Applet Information";
   }
   //Get parameter info
   public String[][] getParameterInfo() {
      return null;
   }

   public static void main(String args[]){
      while(true){}
   }



}

class ControlPanel extends Panel implements ActionListener{
   Button forward, back, start;
   displayApplet applet;
   TextDisplay txt;
   InputPanel inp;
   ProcessReporter processReport;
   String toOut = "";

   static LayoutManager cpLayout = new GridLayout();

   ControlPanel(displayApplet applet, TextDisplay txt, InputPanel inp){
      this.applet = applet;
      this.txt = txt;
      this.inp = inp;
      setLayout(cpLayout);
      start = new Button("Start");
      forward = new Button("Next");
      back = new Button("Back");
      forward.addActionListener(this);
      back.addActionListener(this);
      start.addActionListener(this);
      add(start);
      add(back);
      add(forward);
      back.setEnabled(false);
      forward.setEnabled(false);

   }

   public void actionPerformed(ActionEvent e) {
      if (e.getSource() == back){
           showPrev();
      }
      if (e.getSource() == forward){
         showNext();
      }
      if (e.getSource() == start){
         startDemo();
         back.setEnabled(true);
         forward.setEnabled(true);

         repaint();
      }
    }

    public void setUpReport(String message, String p, String q){
        processReport = new ProcessReporter("" +p, "" +q, "" + 11, message);
    }

    public void showNext(){
       toOut = processReport.getNext();
       txt.setText(toOut);

    }

    public void showPrev(){
       toOut = processReport.getPrev();
       txt.setText(toOut);

    }

    private void startDemo(){
       BigInteger p, q;
       String message;
       message = inp.getMessage();
       p = new BigInteger(getP());
       q = new BigInteger(getQ());


       if( message.length() == 0 ){
          txt.setText("Please input a message");
          return;
       }


       if( (p.intValue() == 0) | (q.intValue() == 0)){
          txt.setText("Please input numbers for both P and Q");
          return;
       }
       else if( !FHMaths.isPrime(p) ){
          while (!FHMaths.isPrime(p)){
             p = new BigInteger(getP());
             inp.setP(FHMaths.generatePrime(p).toString());
          }
       }
       if ( !FHMaths.isPrime(q) ){
          while (!FHMaths.isPrime(q)){
             q = new BigInteger(getQ());
             inp.setQ(FHMaths.generatePrime(q).toString());
          }
       }



       setUpReport(message, p.toString(), q.toString());
       showNext();

    }

    public String getP(){
      String p;
      p = inp.inputP.getText();

      /*
      if(p.compareToIgnoreCase("") == 0){
         return "0";
      }

      for( int i =0; i < p.length(); i++){
         if (!Character.isDigit(p.charAt(i))){
            inp.inputP.setText("");
            return "0";
         }
      }
      */
      return p;
   }

   public String getQ(){
      String q;
      q = inp.inputQ.getText();
      /*
     if(q.compareToIgnoreCase("") == 0){
         return "0";
      }

      for( int i =0; i < q.length(); i++){
         if (!Character.isDigit(q.charAt(i))){
            inp.inputQ.setText("");
            return "0";
         }
      }
      */
      return q;
   }
}

class InputPanel extends Panel{

   public TextField inputP, inputQ, message;
   Label p, q, spacer, m;
   Panel pqPanel, messagePanel;
   static LayoutManager ipLayout = new GridLayout();

   InputPanel(){
      setLayout(new BorderLayout());
      pqPanel = new Panel();
      messagePanel = new Panel();
      pqPanel.setLayout(ipLayout);

      messagePanel.setLayout(new BorderLayout());
      message = new TextField(100);
      m = new Label("Message to encrypt:");
      messagePanel.add("West", m);
      messagePanel.add("Center", message);

      inputP = new TextField(10);
      inputQ = new TextField(10);
      p = new Label("Input P:", 2);
      q = new Label("Input Q:", 2);
      spacer = new Label("");
      pqPanel.add(p);
      pqPanel.add(inputP);
      pqPanel.add(q);
      pqPanel.add(inputQ);
      pqPanel.add(spacer);
      add("North", pqPanel);
      add("South", messagePanel);

   }



   public String getMessage(){
      return message.getText();
   }

   public void setP(String p){
      inputP.setText(p);
   }

   public void setQ(String q){
      inputQ.setText(q);
   }

}

class TextDisplay extends TextArea{

   String toOut;


   TextDisplay(){
      toOut  = "";

      setFont(new Font("Times New Roman", 1, 16));
      setEditable(true);

      setText(toOut);
   }




}










